@@ -9,20 +9,23 @@ __all__ = ["API", "KOI", "Planet", "Star", "LightCurve", "TargetPixelFile"]
 import os
 import re
 import json
-import types
 import shutil
-import urllib
-import urllib2
 import logging
 from itertools import product
 from tempfile import NamedTemporaryFile
 
+import six
+from six.moves import urllib
+
 # Optional dependencies.
 try:
     import pyfits
     pyfits = pyfits
 except ImportError:
-    pyfits = None
+    try:
+        import astropy.io.fits as pyfits
+    except ImportError:
+        pyfits = None
 
 try:
     import fitsio
@@ -90,7 +93,7 @@ class API(object):
 
         # Deal with sort order.
         if sort is not None:
-            if isinstance(sort, types.StringTypes):
+            if isinstance(sort, six.string_types):
                 params["order"] = sort
             else:
                 params["order"] = sort[0]
@@ -99,14 +102,15 @@ class API(object):
 
         # Format the URL in the *horrifying* way that EA needs it to be...
         # they don't un-escape the HTTP parameters!!!
-        payload = ["{0}={1}".format(k, urllib.quote_plus(v, "\"'+"))
+        payload = ["{0}={1}".format(k, urllib.parse.quote_plus(v, "\"'+"))
                    for k, v in params.items()]
 
         # Send the request.
-        r = urllib2.Request(self.ea_url, data="&".join(payload))
-        handler = urllib2.urlopen(r)
+        encoded_data = "&".join(payload).encode("ascii")
+        r = urllib.request.Request(self.ea_url, data=encoded_data)
+        handler = urllib.request.urlopen(r)
         code = handler.getcode()
-        txt = handler.read()
+        txt = handler.read().decode("ascii")
 
         # Hack because Exoplanet Archive doesn't return HTTP errors.
         if int(code) != 200 or "ERROR" in txt:
@@ -140,7 +144,7 @@ class API(object):
 
         # Deal with sort order.
         if sort is not None:
-            if isinstance(sort, types.StringTypes):
+            if isinstance(sort, six.string_types):
                 params["ordercolumn1"] = sort
             else:
                 params["ordercolumn1"] = sort[0]
@@ -148,20 +152,25 @@ class API(object):
                     params["descending1"] = "on"
 
         # Send the request.
-        r = urllib2.Request(self.mast_url.format(category),
-                            data=urllib.urlencode(params))
-        handler = urllib2.urlopen(r)
+        encoded_data = urllib.parse.urlencode(params).encode("ascii")
+        r = urllib.request.Request(self.mast_url.format(category),
+                                   data=encoded_data)
+        handler = urllib.request.urlopen(r)
         code = handler.getcode()
-        txt = handler.read()
+        txt = handler.read().decode("ascii")
         if int(code) != 200:
-            full_url = handler.geturl() + "?" + urllib.urlencode(params)
+            full_url = handler.geturl() + "?" + urllib.parse.urlencode(params)
             raise APIError(code, full_url, txt)
 
+        # Check for no rows found.
+        if "no rows" in txt:
+            return []
+
         # Parse the JSON.
         try:
             result = json.loads(txt)
         except ValueError:
-            full_url = handler.geturl() + "?" + urllib.urlencode(params)
+            full_url = handler.geturl() + "?" + urllib.parse.urlencode(params)
             raise APIError(code, full_url,
                            "No JSON object could be decoded.\n" + txt)
 
@@ -288,7 +297,7 @@ class API(object):
                              .format(kepid))
         return stars[0]
 
-    def _data_search(self, kepler_id, short_cadence=True):
+    def _data_search(self, kepler_id=None, short_cadence=True, **params):
         """
         Run a generic data search on MAST to return a list of dictionaries
         describing the data products.
@@ -300,21 +309,22 @@ class API(object):
             A boolean flag that determines whether or not the short cadence
             data should be included.
 
+        :param ** params:
+            Any other search parameters.
+
         """
-        params = {"ktc_kepler_id": kepler_id}
+        if kepler_id is not None:
+            params["ktc_kepler_id"] = kepler_id
         if not short_cadence:
             params["ktc_target_type"] = "LC"
 
         data_list = self.mast_request("data_search",
                                       adapter=mast.dataset_adapter,
                                       **params)
-        if not len(data_list):
-            raise ValueError("No data files found for: '{0}'"
-                             .format(kepler_id))
         return data_list
 
-    def light_curves(self, kepler_id, short_cadence=True, fetch=False,
-                     clobber=False):
+    def light_curves(self, kepler_id=None, short_cadence=True, fetch=False,
+                     clobber=False, **params):
         """
         Find the set of light curves associated with a KIC target.
 
@@ -333,15 +343,18 @@ class API(object):
             A boolean flag that determines whether or not the data file should
             be overwritten even if it already exists.
 
+        :param ** params:
+            Other search parameters to be passed to the MAST data search.
+
         """
         lcs = [LightCurve(self, d) for d in self._data_search(kepler_id,
-               short_cadence=short_cadence)]
+               short_cadence=short_cadence, **params)]
         if fetch:
             [l.fetch(clobber=clobber) for l in lcs]
         return lcs
 
-    def target_pixel_files(self, kepler_id, short_cadence=True, fetch=False,
-                           clobber=False):
+    def target_pixel_files(self, kepler_id=None, short_cadence=True,
+                           fetch=False, clobber=False, **params):
         """
         Find the set of target pixel files associated with a KIC target.
 
@@ -360,9 +373,12 @@ class API(object):
             A boolean flag that determines whether or not the data file should
             be overwritten even if it already exists.
 
+        :param ** params:
+            Other search parameters to be passed to the MAST data search.
+
         """
         tpfs = [TargetPixelFile(self, d) for d in self._data_search(kepler_id,
-                short_cadence=short_cadence)]
+                short_cadence=short_cadence, **params)]
         if fetch:
             [l.fetch(clobber=clobber) for l in tpfs]
         return tpfs
@@ -413,7 +429,7 @@ class Model(object):
     def __init__(self, api, params):
         self.api = api
         self.params = params
-        for k, v in params.iteritems():
+        for k, v in params.items():
             setattr(self, k, v)
 
         self._name = self._id.format(**params)
@@ -660,8 +676,8 @@ class _datafile(Model):
         # Fetch the remote file.
         url = self.url
         logging.info("Downloading file from: '{0}'".format(url))
-        r = urllib2.Request(url)
-        handler = urllib2.urlopen(r)
+        r = urllib.request.Request(url)
+        handler = urllib.request.urlopen(r)
         code = handler.getcode()
         if int(code) != 200:
             raise APIError(code, url, "")
@@ -743,7 +759,7 @@ class LightCurve(_datafile):
                     alpha=0.6)
             ax.set_xlim(xlim)
             ax.set_ylim(np.min(f1), np.max(f1))
-            ax.annotate("relative "+nm+" [ppm]",
+            ax.annotate("relative " + nm + " [ppm]",
                         xy=(1, 1), xycoords="axes fraction",
                         xytext=(-3, -3), textcoords="offset points",
                         horizontalalignment="right", verticalalignment="top")
